/* SCCS ID: @(#)dev.x	1.17 1/14/94 */
/*
 * dev.x,v
 * Revision 1.8  2005/03/01 22:26:27  soliday
 * Updated to work with Epics/Base 3.14.7
 *
 * Revision 1.7  2002/08/02 15:38:30  jba
 * Added license information.
 *
 * Revision 1.6  2000/10/24 20:45:11  jba
 * Fixed comments and made some ansi c changes.
 *
 * Revision 1.5  1996/10/02 18:51:26  saunders
 * Restored use of lex and yacc, since flex and antelope aren't
 * handling the grammar for some reason. Also touched up some
 * compiler warnings.
 *
# Revision 1.4  1995/10/04  21:41:23  jba
# Portability changes.
#
# Revision 1.3  1995/03/03  23:00:33  jba
# Changes for solaris port
#
# Revision 1.2  1994/05/11  20:17:47  saunders
# First fully functional version added to cvs addon environment.
# */
/*************************************************************************/
/* FILE:      dev.x                                                      */
/* Author:    Claude Saunders                                            */
/* Purpose:   rpcgen input file                                          */
/*            Device server protocol and interface definition.           */
/*            Rpcgen takes this file and generates server, client, and   */
/*            XDR (data format conversion) code as well as a header file */
/*                                                                       */
/* Mods:      login mm/dd/yy: description                                */
/*************************************************************************/

/* Error return codes for resolve_result.myerrno */
const OK = 0;
const NOSUCHDEV = 1;
const NOSUCHMSG = 2;

const MAXNAMELENDEV = 80;

typedef struct devicenode *devicelist;
typedef struct request_struct requesttype;
typedef struct namenode *namelist;

struct namenode {
	string dname<>;
	namelist next;
};

struct request_struct {
	string dname<>;
	string mesg<>;
};

struct devicenode {
	string dname<>;
	int type;
	string msg<>;
	string pvname<>;
	int dir;
	int nargs;
	int datatype;
	string def<>;
	devicelist next;
};

struct resolve_result {
	devicelist reply;
	int myerrno;
};

/* Definition of external services provided  */
/* (ie. available remote procedure calls)    */
program DEVICE_PROG {
	version DEVICE_VERS {
		resolve_result RESOLVE_DEVICE(requesttype) = 1;
		string SCAN_COMPOSITE_DEFS(string) = 2;
		namelist DEVICE_COUNT(string) = 3;
		namelist DEVICE_MATCH(string) = 4;
		resolve_result DEVICE_INFO(string) = 5;
	} = 1;
} = 0x31234567;    /* arbitrary program number in user-defined ONC RPC range */

/* Any code here which begins with % is blindly placed in the client and */
/* server stubs. The ifdef RPC_SVC causes it to be placed only in the    */
/* server stub. This replacement of the main() function                  */
/* is neccessary to allow us to first scan device definitions            */
/* before we begin to service requests. This technique means rpcgen can  */
/* still be used to automatically generate client and server stubs.      */

/* There is a danger here in that the main() function (normally)         */
/* generated by rpcgen may change, while this replacement doesn't.       */
/* More likely is that rpcgen will improve in its ability to support     */
/* initialization code. Keep an eye out for a better RPC protocol        */
/* compiler. The DCE rpc protocol compiler is a possibility.             */

#ifdef RPC_SVC
%#if defined(SOLARIS)
%#define PORTMAP
%#include <rpc/rpc.h>
%#endif
%#include <stdio.h>
%#include <sys/ioctl.h>
%#include <sys/types.h>
%#include <sys/socket.h>
%#include <syslog.h>
%#include <unistd.h>
%#include "scan_defs.h"
%
%int logerror(char *msg1, char *msg2);
%static void device_prog_1();
%#if defined(SOLARIS) || defined(SGI)
%static void _msgout();
%#endif
%static int _rpcpmstart;
%static int _rpcfdtype;
%
%int main(int argc, char *argv[])
%{
%	register SVCXPRT *transp;
%	int sock;
%	int proto;
%	struct sockaddr_in saddr;
%	unsigned int asize = sizeof(saddr);
%
%	if (getsockname(0, (struct sockaddr *)&saddr, &asize) == 0) {
%		unsigned int ssize = sizeof(int);
%
%		if (saddr.sin_family != AF_INET)
%			exit(1);
%		if (getsockopt(0, SOL_SOCKET, SO_TYPE,
%				(char *)&_rpcfdtype, &ssize) == -1)
%			exit(1);
%		sock = 0;
%		_rpcpmstart = 1;
%		proto = 0;
%		openlog("dev", LOG_PID, LOG_DAEMON);
%	} else {
%#ifndef RPC_SVC_FG
%		int i, pid;
%
%		pid = fork();
%		if (pid < 0) {
%			perror("cannot fork");
%			exit(1);
%		}
%		if (pid)
%			exit(0);
%		for (i = 0 ; i < 20; i++)
%			(void) close(i);
%		i = open("/dev/console", 2);
%		(void) dup2(i, 1);
%		(void) dup2(i, 2);
%#ifdef SOLARIS
%		setsid();
%#else
%		i = open("/dev/tty", 2);
%		if (i >= 0) {
%			(void) ioctl(i, TIOCNOTTY, (char *)NULL);
%			(void) close(i);
%		}
%#endif
%		openlog("dev", LOG_PID, LOG_DAEMON);
%#endif
%		sock = RPC_ANYSOCK;
%		(void) pmap_unset(DEVICE_PROG, DEVICE_VERS);
%	}
%
%	if ((_rpcfdtype == 0) || (_rpcfdtype == SOCK_STREAM)) {
%		transp = (SVCXPRT *)svctcp_create(sock, 0, 0);
%		if (transp == NULL) {
%			_msgout("cannot create tcp service.");
%			exit(1);
%		}
%		if (!_rpcpmstart)
%			proto = IPPROTO_TCP;
%		if (!svc_register(transp, DEVICE_PROG, DEVICE_VERS, device_prog_1, proto)) {
%			_msgout("unable to register (DEVICE_PROG, DEVICE_VERS, tcp).");
%			exit(1);
%		}
%	}
%
%	if (transp == (SVCXPRT *)NULL) {
%		_msgout("could not create a handle");
%		exit(1);
%	}
%/***** NOTE: this is custom initialization code added to generic main() */
%	if (scan_defs() != 0) {
%	  logerror("init","scan of initial defs failed");
%         exit(1);
%	}
%/************************************************************************/
%	svc_run();
%	_msgout("svc_run returned");
%	return(1);
%	/* NOTREACHED */
%}
%
#endif 
