/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
/* SCCS ID: @(#)dev_count.c	1.3 9/27/93 */
/*************************************************************************
 * FILE:      dev_count.c
 * Author:    Claude Saunders
 * Purpose:   Implements the devCount device access library call. The device
 *            access library invokes the device_count_1() remote procedure
 *            call which is serviced here.
 *
 * Mods:      login mm/dd/yy: description
 *************************************************************************/
#include <stdio.h>
#include <rpc/rpc.h>
#include <string.h>
#include <stdlib.h>
#include "dev.h"
#include "scan_defs.h"

/*************************************************************************
 * FUNCTION : device_count_1()
 * PURPOSE  : Given a device name, return a singly linked list of the
 *            constituent atomic device names. If dname is itself atomic,
 *            the list will have only one node. If dname is composite, there
 *            will be one node for each constituent atomic device. Back
 *            in the device access library, this linked list is traversed
 *            to get a count of the number of elements.
 *
 * ARGS in  : dname: string containing device name
 * ARGS out : none
 * GLOBAL   : 
 * RETURNS  : singly linked list of device names (see RPC protocol file dev.x)
 ************************************************************************/
namelist *device_count_1(dname)
     char **dname;
{
  PDEV pdev;                  /* device header node */
  PPVMSG ppvmsg;              /* ptr to linked list of supported messages */
  PMSG pmsg;                  /* ptr to message specific information node */
  static namelist res;        /* RPC return MUST be static ! */
  namelist list = NULL;
  namelist prevlist = NULL;
  namelist listhead = NULL;

  xdr_free(xdr_namelist,(char *)&res);   /* free up from previous reply */
  pdev = dev_find(*dname);               /* find dev in hash table */

  if (pdev == NULL)
    res = NULL;
  else {
    if (pdev->type == COMPOSITE)  /* skip over header node */
      pdev = pdev->next;
    while (pdev != NULL) {
      list = (namelist)malloc(sizeof(struct namenode));
      if (prevlist == NULL)
	listhead = list;
      else
	prevlist->next = list;
      list->next = NULL;
      prevlist = list;
      list->dname = strdup(pdev->name);
      pdev = pdev->next;
    }
  }

  res = listhead;
  return(&res);
}

