/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
/* SCCS ID: @(#)dev_info.c	1.5 1/13/94 */
/*************************************************************************
 * FILE:      dev_info.c
 * Author:    Claude Saunders
 * Purpose:   Implements the devInfo device access library call. The device
 *            access library invokes the device_info_1() remote procedure
 *            call which is serviced here.
 *
 * Mods:      login mm/dd/yy: description
 *************************************************************************/
#include <stdio.h>
#include <rpc/rpc.h>
#include <string.h>
#include <stdlib.h>
#include "dev.h"
#include "scan_defs.h"

/*************************************************************************
 * FUNCTION : device_info_1()
 * PURPOSE  : Given an atomic or composite device name, this function
 *            returns a singly linked list of devicenode structs (see
 *            RPC interface definition file dev.x). The devicenode struct
 *            contains all information the device server has about that
 *            device. If dname is an atomic device, there will be one node
 *            on the list for each message supported by that device
 *            class. If dname is a composite device, there will be a node
 *            for each message of each constituent atomic device.
 *
 * ARGS in  : dname: string containing device name
 * ARGS out : none
 * GLOBAL   : none
 * RETURNS  : resolve_result.myerrno indicates whether resolve_result.reply
 *            is valid or not. The linked list is actually the .reply field.
 *            This technique is shown in the RPC manuals.
 ************************************************************************/
resolve_result *device_info_1(dname)
     char **dname;
{
  PDEV pdev;                  /* device header node */
  PPVMSG ppvmsg;              /* ptr to linked list of supported messages */
  PMSG pmsg;                  /* ptr to message specific information node */
  devicelist *dlptr;          /* for building up resolve_result.reply     */
  devicelist dl, prevdl;
  static resolve_result res;  /* RPC return MUST be static ! */

  xdr_free(xdr_resolve_result,(char *)&res); /* free up reply from last call */

  pdev = dev_find(*dname);                 /* find dev in server hash table */
  res.reply = NULL;       /* initialize reply list */
  res.myerrno = OK;
  dlptr = &res.reply;     /* get addr of reply list */

  if (pdev == NULL) {           /* can't find requested device in hash table */
    res.myerrno = NOSUCHDEV;
    return(&res);
  }

  if (pdev->type == COMPOSITE)  /* skip over header node */
    pdev = pdev->next;

  /* For each device and message, add a node to reply list. */
  while (pdev != NULL) {      /* for each device */
    ppvmsg = pdev->pvmsgh;
    while (ppvmsg != NULL) {  /* for each message */
      pmsg = ppvmsg->msgh;
      dl = (devicenode *)malloc(sizeof(devicenode));
      if (*dlptr == NULL)
	*dlptr = dl;
      else
	prevdl->next = dl;
      prevdl = dl;
      
      dl->next = NULL;                       /* load up node with info */
      dl->dname = strdup(pdev->name);
      dl->type = pdev->type;
      dl->msg = strdup(pmsg->name);
      dl->pvname = strdup(ppvmsg->pvname);
      dl->dir = pmsg->dir;
      dl->nargs = pmsg->nargs;
      dl->datatype = pmsg->type;
      dl->def = strdup(pmsg->def);
      
      ppvmsg = ppvmsg->next;    /* advance to next message */
    }
    pdev = pdev->next;          /* advance to next dev */
  }
  
  return(&res);
}

